import React from 'react';
import { createGame, gameInfo, joinGame, getGuesses, sendGuess } from "./services/ggsh.service";
import GamePanel from "./components/game-panel.component";
import ApiInfo from "./components/api-info.component";
import PlayerScores from "./components/player-scores.component";
import './App.css';
import RoundSummary from './components/round-summary.component';


class Game extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      playerName: "testing",
      loading: false,
      gameId: null,
      currentRound: null,
      targetPoint: null,
      selectedPoint: null,
      betweenRounds: false,
      lastScore: null,
      totalScore: null,
      playerScores: null,
    }
  }

  async handleGameCreate() {
    this.setState({loading: true});
    const { playerName } = this.state;
    const gameId = await createGame(playerName, 300);
    this.setState({ gameId });
    await this.updateRoundState();
  }

  handleRoundEnd(coords, currentRound) {
    const targetPoint = coords[currentRound];
    this.setState({
      loading: false,
      currentRound,
      targetPoint,
      selectedPoint: null,
      betweenRounds: false,
    });
  }

  handleGameEnd(playerScores) {
    this.setState({
      loading: false,
      gameId: null,
      currentRound: null,
      targetPoint: null,
      selectedPoint: null,
      betweenRounds: false,
      playerScores,
    });
  }

  async updateRoundState() {
    this.setState({ loading: true })
    const { gameId, playerName } = this.state;
    const { currentRound } = await getGuesses(gameId, playerName);
    const { coords, players } = await gameInfo(gameId);
    if (currentRound) {
      this.handleRoundEnd(coords, currentRound);
    } else {
      this.handleGameEnd(players);
    }
  }

  async handleSubmitGuess() {
    const {
      gameId,
      playerName,
      currentRound,
      selectedPoint
    } = this.state;
    this.setState({ loading: true });
    const { score, totalScore } = await sendGuess(gameId, playerName, currentRound, selectedPoint);
    this.setState({
      loading: false,
      betweenRounds: true,
      lastScore: score,
      totalScore
    });
  }

  render() {
    const {
      loading,
      currentRound,
      selectedPoint,
      targetPoint,
      betweenRounds,
      lastScore,
      totalScore,
      playerScores,
    } = this.state;

    // TODO eventually make state transitions more clear

    if (loading) {
      return <p>Loading...</p>
    }

    if (betweenRounds) {
      return <RoundSummary
        roundNum={currentRound}
        score={lastScore}
        totalScore={totalScore}
        onAdvanceState={() => this.updateRoundState()}
        buttonText={currentRound === "5" ? "View Summary" : "Next Round"}
      />
    }

    if (playerScores) {
      return <PlayerScores
        scores={playerScores}
        onReturnToStart={() => this.setState({ playerScores: null })}
      />
    }

    if (!targetPoint) {
      return (
        <div>
          <p>No game in progress!</p>
          <button onClick={() => this.handleGameCreate()}>Create New Game</button>
        </div>
      );
    }

    return <GamePanel
      onSelectPoint={latLng => this.setState({selectedPoint: latLng})}
      onSubmitGuess={() => this.handleSubmitGuess()}
      streetViewPoint={targetPoint}
      selectedPoint={selectedPoint}
    />
  }

}

const App = () => {
  return (
    <div className="App" style={{
      display: "flex",
      flexDirection: "column",
      justifyContent: "space-between",
      height: "100vh",
    }}>
      <div>
        <p>TerrAssumptions!</p>
        <hr/>
      </div>
      <Game style={{flex: 1}}/>
      <div>
        <hr/>
        <ApiInfo/>
      </div>
    </div>
  );
}

export default App;
